////////////////////////////////////////////////////////////////////////
//
//      Fichier: Barre_impl.h
//       Auteur: Stephane Drouin
//     Creation: 20/03/01 12:07:54
// Modification: 04/04/01
//               Ajout de la methode Detruire()
//  Description: Implantation d'une barre distribuee
//
////////////////////////////////////////////////////////////////////////

#ifndef __BARRE_IMPL_H__
#define __BARRE_IMPL_H__

#include <Reacteur_skel.h>

class Barre_impl : public POA_Barre, public PortableServer::RefCountServantBase {

public:
	Barre_impl() : grille(0) {}
	~Barre_impl();

	// Initialisation de la simulation et verification des parametres
    virtual void Initialisation(const ParametresSimulation& par)
        throw(MauvaisParametres,CORBA::SystemException);

	// Le client doit detruire le servant apres utilisation
	virtual void Detruire() throw(CORBA::SystemException);

	// Calcul d'une iteration de 'pas' secondes, avec gradients voisins 1 et 2
    virtual void Iteration(TimeType pas, const Barre::TempArray temp_cal)
        throw(ErreurCalcul,CORBA::SystemException);

	// Obtenir la temperature maximale et les gradients de temperature
    virtual TempType temp_max() throw(CORBA::SystemException);
    virtual Barre::TempArray_slice* grad_temp() throw(CORBA::SystemException);
	

private:
	TempType *grille;			// Temperature des points de la barre
	Barre::TempArray gradient;	// Gradients de temperature
	TempType maxtemp;			// Temperature maximale de la barre
	ParametresSimulation p;		// Tous les parametres de simulation
};

#endif
