////////////////////////////////////////////////////////////////////////
//
//      Fichier: Canal_impl.cpp
//       Auteur: Stephane Drouin
//     Creation: 21/02/01 21:46:50
// Modification: 28/03/01
//  Description: See Canal_impl.hpp
//
////////////////////////////////////////////////////////////////////////

#include <iostream.h>
#include <OB/CORBA.h>
#include <Canal_impl.h>

////////////////////////////////////////////////////////////////////////
// Destructeur d'un canal
////////////////////////////////////////////////////////////////////////
Canal_impl::~Canal_impl() { 
	cout << "Destruction d'un canal..." << endl;	// Pour signaler la vie!
}

////////////////////////////////////////////////////////////////////////
// Le client doit detruire le servant apres utilisation
////////////////////////////////////////////////////////////////////////
void Canal_impl::Detruire() {
	// Code tire de la doc de ORBacus, page 91
	PortableServer::POA_var poa = _default_POA();
	PortableServer::ObjectId_var id = poa->servant_to_id(this);
	poa->deactivate_object(id);

	// When the reference count reaches a value of of the zero,
	// the _remove_ref code will issue the call delete this.
	// http://phoenix.liu.edu/~mdevi/corba/RefCountServantBase.htm
	_remove_ref();
}

////////////////////////////////////////////////////////////////////////
// Initialisation de la simulation et verification des parametres
////////////////////////////////////////////////////////////////////////
void Canal_impl::Initialisation(const ParametresSimulation& par) {

	cout << "Initialisation d'un canal" << endl;	// Pour signaler la vie!

	// Validation des parametres de simulation touchant ce canal
	p = par;
	if (p.cvit<0) throw MauvaisParametres(p,"Vitesse du caloporteur negative");
	if (p.tempin<0) throw MauvaisParametres(p,"Temperature d'entree negative");
	if (p.diffusion<0) throw MauvaisParametres(p,"Diffusion de chaleur negative");
	if (p.chspecc<=0) throw MauvaisParametres(p,"chspecc doit etre positif");
	if (p.densc<=0) throw MauvaisParametres(p,"densc doit etre positif");
	if (p.longb<=0) throw MauvaisParametres(p,"Longueur barre doit etre positive");
	if (p.largc<=0) throw MauvaisParametres(p,"Largeur canal doit etre positive");
  
	// Temperature moyenne initiale de ce canal
	tc = p.tempin;
}

////////////////////////////////////////////////////////////////////////
// Obtention de la temperature moyenne du canal
////////////////////////////////////////////////////////////////////////
TempType Canal_impl::temp_moy() { return tc; }

////////////////////////////////////////////////////////////////////////
// Calcul de la nouvelle temperature moyenne du canal
////////////////////////////////////////////////////////////////////////
void Canal_impl::Iteration(TimeType pas, TempType grad_temp_1,
                                    TempType grad_temp_2) {
	TempType deltaT;	// Difference de temperature calculee par formule
	deltaT = (p.diffusion*(grad_temp_1+grad_temp_2)/(p.chspecc*p.densc*p.largc)
		+ 2*p.cvit*(p.tempin-tc)/(p.longb))*pas;

	tc += deltaT;		// Nouvelle temperature

	if (tc < 0) {
		throw ErreurCalcul("Temperature negative dans un canal!");
	}
}