#include <OB/CORBA.h>
#include <Reacteur.h>
#include <fstream.h>
#include <iostream.h>
#include <string.h>
#include <conio.h>
#include <stdlib.h>
#include "Reacteur_impl.h"

#define PS_DEF_CVIT       4		// Parametres de simulation par defaut !
#define PS_DEF_CHIN       5e8
#define PS_DEF_TEMPIN     400
#define PS_DEF_DIFFUSION  2.2
#define PS_DEF_CHSPECB    220
#define PS_DEF_CHSPECC    698
#define PS_DEF_DENSB      1e4
#define PS_DEF_DENSC      800
#define PS_DEF_LONGB      0.5
#define PS_DEF_LARGB      0.01
#define PS_DEF_LARGC      0.02
#define PS_DEF_TFUS       2800
#define PS_DEF_NGRILLES   10
#define PS_DEF_NBARRES    3

istream& operator>> (istream&, ParametresSimulation&);	// Lit le fichier config
ostream& operator<< (ostream&, ParametresSimulation&);	// Affiche la config
TimeType Affiche(Reacteur_impl&, TimeType &); // Affiche reacteur, obtient commande

int run(CORBA::ORB_ptr orb) {		// Corps de l'application!
	int i;

	TimeType pas = 0.1;				// Le pas de la simulation
	ParametresSimulation ps;		// Lecture des parametres de simulation
	ifstream inps("config.txt",ios::in);
	inps >> ps;
	inps.close();

	cout << ps << endl;				// Ecran d'introduction de la simulation
	cout << "Appuyez sur une touche pour demarrer la simulation..." << endl;
	int tmp = getch();				// Attend touche pour demarrer

	Reacteur_impl reacteur(orb);	// Creation du reacteur

	try {
		reacteur.Initialisation(ps);		// Initialisation du reacteur
		TimeType nbsec;	// Tant que l'utilisateur le veut, la simulation a lieu!
		while (nbsec=Affiche(reacteur,pas)) {	// Avancer de 'nbsec' secondes
			for (i=0;i<nbsec/pas;++i) reacteur.Iteration(pas);
		}
	} catch (MauvaisParametres & e) {		// Erreur d'init ou parametres
		cerr << endl << "Erreur d'initialisation: " << e.message << endl;
		getch();
		return EXIT_FAILURE;
	} catch (Reacteur::MauvaisIndice & e) {	// Acces incorrect a element d'un array
		cerr << endl << "Mauvais indices: " << e.message << " (";
		for (i=0;i<e.indices.length()-1;++i) cerr << e.indices[i] << ", ";
		cerr << e.indices[i] << ")" << endl;
		getch();
		return EXIT_FAILURE;
	} catch (ErreurCalcul &e) {			// Hmmm, probablement temperature negative
		cerr << endl << "Erreur de calcul: " << e.message << endl;
		getch();
		return EXIT_FAILURE;
	} catch (Reacteur::Fusion & e) {		// Fusion du reacteur! Fin de simulation
		cerr << endl << "Fusion du reacteur! " << e.details.length()
			 << " barres en fusion!" << endl;
		for (i=0;i<e.details.length();++i) {	// Affiche barres en fusion
			cerr << "\tBarre (" << e.details[i].i << ", " << e.details[i].j
				 << ") : " << e.details[i].temp << " kelvins a "
				 << e.details[i].instant << " secondes." << endl;
		}
		pas = 0;
		Affiche(reacteur,pas);			// Affiche etat final du reacteur
		return EXIT_SUCCESS;
	}
	
	return EXIT_SUCCESS;
}

////////////////////////////////////////////////////////////////////////
// Lecture d'un fichier de configuration pour la simulation
////////////////////////////////////////////////////////////////////////
istream& operator>> (istream& in, ParametresSimulation& ps){
	// Place les valeurs par defaut de chaque parametre
	// Au cas ou ils ne seraient pas modifies par le fichier de conf.
	ps.cvit = PS_DEF_CVIT;
	ps.chin = PS_DEF_CHIN;
	ps.tempin = PS_DEF_TEMPIN;
	ps.diffusion = PS_DEF_DIFFUSION;
	ps.chspecb = PS_DEF_CHSPECB;
	ps.chspecc = PS_DEF_CHSPECC;
	ps.densb = PS_DEF_DENSB;
	ps.densc = PS_DEF_DENSC;
	ps.longb = PS_DEF_LONGB;
	ps.largb = PS_DEF_LARGB;
	ps.largc = PS_DEF_LARGC;
	ps.Tfus = PS_DEF_TFUS;
	ps.ngrilles = PS_DEF_NGRILLES;
	ps.nbarres = PS_DEF_NBARRES;

	// Fait la lecture du fichier de configuration en ignorant
	// les commentaires et en assignant les valeurs selon les mot-cles,
	// qui sont les noms des variables.
	char tmp, ligne[250], kword[100];
	while ( in.get(tmp) ) {
		if (tmp=='%'||tmp==' ') in.getline(ligne,250);  // Ignore les commentaires
		else if (tmp != '\n') {
			in.putback(tmp);
			in >> kword;								// Lecture du mot cle
			if (!strcmp(kword,"cvit")) in >> ps.cvit;	//  puis de la valeur
			else if (!strcmp(kword, "chin")) in >> ps.chin;
			else if (!strcmp(kword, "tempin")) in >> ps.tempin;
			else if (!strcmp(kword, "diffusion")) in >> ps.diffusion;
			else if (!strcmp(kword, "chspecb")) in >> ps.chspecb;
			else if (!strcmp(kword, "chspecc")) in >> ps.chspecc;
			else if (!strcmp(kword, "densb")) in >> ps.densb;
			else if (!strcmp(kword, "densc")) in >> ps.densc;
			else if (!strcmp(kword, "longb")) in >> ps.longb;
			else if (!strcmp(kword, "largb")) in >> ps.largb;
			else if (!strcmp(kword, "largc")) in >> ps.largc;
			else if (!strcmp(kword, "Tfus")) in >> ps.Tfus;
			else if (!strcmp(kword, "ngrilles")) in >> ps.ngrilles;
			else if (!strcmp(kword, "nbarres")) in >> ps.nbarres;
			else cerr << "Lecture des parametres. INVALIDE: " << kword << endl;
			in.getline(ligne,250);          // Ignore le reste de la ligne
		}
	}

	return in;
}

////////////////////////////////////////////////////////////////////////
// Affichage de la configuration de simulation
////////////////////////////////////////////////////////////////////////
ostream& operator<< (ostream& out, ParametresSimulation& ps){
	out << "Valeurs des parametres de simulation" << endl;
	out << "\tcvit      : " << ps.cvit << endl;
	out << "\tchin      : " << ps.chin << endl;
	out << "\ttempin    : " << ps.tempin << endl;
	out << "\tdiffusion : " << ps.diffusion << endl;
	out << "\tchspecb   : " << ps.chspecb << endl;
	out << "\tchspecc   : " << ps.chspecc << endl;
	out << "\tdensb     : " << ps.densb << endl;
	out << "\tdensc     : " << ps.densc << endl;
	out << "\tlongb     : " << ps.longb << endl;
	out << "\tlargb     : " << ps.largb << endl;
	out << "\tlargc     : " << ps.largc << endl;
	out << "\tTfus      : " << ps.Tfus << endl;
	out << "\tngrilles  : " << ps.ngrilles << endl;
	out << "\tnbarres   : " << ps.nbarres << endl;
	return out;
}

////////////////////////////////////////////////////////////////////////
// Affichage de l'etat d'un reacteur
// Retourne le nombre de secondes desirees pour avancer la simulation
////////////////////////////////////////////////////////////////////////
TimeType Affiche(Reacteur_impl& reacteur, TimeType &pas) {
	IndiceType i,j;
	IndiceType nbarres = reacteur.NombreDeBarres();

	// Affichage de toutes les temperatures maximales et moyenne avec tabulation
	// pour representation spaciale correcte des valeurs...
	for (i=1;i<=nbarres+1;++i) {	// Traiter toutes les rangees de barres/canal
		cout << endl;
		// premiere ligne de i: caloporteurs indice k=2:
		// [] [canal] [] [canal] ...
		for (j=1;j<=nbarres;++j) {
			cout << "\t" << (int)reacteur.calop(i,j,2) << "\t" ;
		}
		
		// deuxieme ligne de i: caloporteurs indice k=1 et barres:
		// [canal] [barre] [canal]...
		if (i==nbarres+1) continue;	// sans objet si derniere ligne
		cout << endl;
		for (j=1;j<=nbarres+1;++j) {
			cout << (int)reacteur.calop(i,j,1) << "\t";
			if (j==nbarres+1) continue;	// s.o. si derniere colonne
			cout << (int)reacteur.temp(i,j) << "\t";
		}
	}

	// Affichage des instruction pour la suite des operations
	cout << endl << endl << "Temps de simulation: " << reacteur.TempsSimulation()
		 << " secondes." << endl;

	// Affichage final de l'etat du reacteur, probablement apres une fusion :)
	if (pas == 0) {
		getch();
		return 0;
	}

	while (1) {	// Traiter toutes les commandes avant de poursuivre ou arreter
		cout << "Appuyez sur '+'/'-' pour augmenter ou diminuer le pas "
			 << "de 0.01 (pas = " << pas << ")." << endl
			 << "Appuyez sur '1', '2',... '0' pour avancer de 1, 2,... 10 secondes."
			 << endl
			 << "Appuyez sur 'Q' pour terminer ou une autre touche pour "
			 << "avancer d'un pas." << endl;

		int tmp = getch();								// Obtention d'une touche
		if (tmp == 'Q' || tmp == 'q') return 0;				// Quitter!
		else if (tmp == '-') { if (pas >0.01) pas-=0.01; }	// Diminuer le pas
		else if (tmp == '+') pas+= 0.01;					// Augmenter le pas
		else if (tmp>='1' && tmp<='9') return (tmp-'0');	// Saut de x sec
		else if (tmp == '0') return 10;						// Saut de 10 sec.
		else break;
	}
    return pas;	// On veut poursuivre, avec un saut de 1 pas.
}
