////////////////////////////////////////////////////////////////////////
//
//      Fichier: Reacteur_impl.hpp
//       Auteur: Stephane Drouin
//     Creation: 07/02/01 14:58:54
// Modification: 28/02/01 20:18:41
//               Ajout de fonctions d'acces aux pointeurs de barres/calop
// Modification: 28/03/01
//				 Implantation du reacteur dans le contexte CORBA.
//  Description: Reacteur nucleaire
//
////////////////////////////////////////////////////////////////////////

#ifndef __REACTEUR_IMPL_HPP__
#define __REACTEUR_IMPL_HPP__

#include <OB/CORBA.h>
#include <Reacteur.h>

class Reacteur_impl {
	// Ceci n'a pas a etre un objet corba, car il s'execute dans le client
	// et n'est utilise que par celui-ci...
	//: public POA_Reacteur,	public PortableServer::RefCountServantBase {

public:
	Reacteur_impl(CORBA::ORB_ptr _orb);
	~Reacteur_impl();

	// Initialisation de la simulation et recuperation des IORs sur barres
    virtual void Initialisation(const ParametresSimulation& par)
        throw(MauvaisParametres,CORBA::SystemException);

	// Obtentation de la temperature d'une barre / d'un canal caloporteur
    virtual TempType temp(IndiceType i,IndiceType j)
        throw(Reacteur::MauvaisIndice,CORBA::SystemException);
    virtual TempType calop(IndiceType i,IndiceType j,IndiceType k)
        throw(Reacteur::MauvaisIndice,CORBA::SystemException);

	// Calcul d'une iteration avec un pas de 'pas' secondes
    virtual void Iteration(TimeType pas)
        throw(Reacteur::Fusion,CORBA::SystemException);

	// Duree de la simulation depuis le debut / dimension du reacteur
	virtual TimeType TempsSimulation() throw(CORBA::SystemException);
    virtual IndiceType NombreDeBarres() throw(CORBA::SystemException);

private:
	// Obtention d'un pointeur sur une barre / un canal (validation des indices)
	Barre_var get_barre(IndiceType i, IndiceType j) const;
	Canal_var get_canal(IndiceType i, IndiceType j, IndiceType k) const;

	Barre_var* barres;		// Tableau de barres de combustible
	Canal_var* canaux;		// Tableau de canaux de caloporteur
	TimeType time;			// Temps de simulation

	FabriqueDeCanaux_var fabCanaux;	// Fabrique de canaux maitre
	FabriqueDeBarres_var fabBarres;	// Fabrique de barres maitre

	CORBA::ORB_ptr orb;		// Pointeur sur l'orb pour acces aux barres

	ParametresSimulation p;	// Tous les parametres de simulation
};

#endif