#include <OB/CORBA.h>
#include <OB/CosNaming.h>
#include <FabriqueDeCanaux_impl.h>
#include <FabriqueDeBarres_impl.h>
#include <stdio.h>

using namespace std;

int run(CORBA::ORB_ptr orb) {		// Corps de l'application!
	CORBA::Object_var obj;

	// Obtention du service de nommage
	try {
		obj = orb->resolve_initial_references("NameService");
	} catch (CORBA::ORB::InvalidName &) {
		cerr << "Service de nomage inconnu" << endl;
		return EXIT_FAILURE;
	}

	if(CORBA::is_nil(obj.in())) {
		cerr << "'NameService' est une rfrence d'objet nil" << endl;
		return EXIT_FAILURE;
	}
	CosNaming::NamingContext_var nsRef = CosNaming::NamingContext::_narrow(obj);
	if (CORBA::is_nil(nsRef.in())) {
		cerr << "Le service inicial 'NameService' n'est pas un NamingContext";
		return EXIT_FAILURE;
	}

	// Initialisation du POA
	obj = orb -> resolve_initial_references("RootPOA");
	PortableServer::POA_var rootPoa = PortableServer::POA::_narrow(obj);
	PortableServer::POAManager_var manager = rootPoa -> the_POAManager();

	// Publication de l'IOR de la fabrique de canaux
	FabriqueDeCanaux_impl * fabCanImpl = new FabriqueDeCanaux_impl;
	FabriqueDeCanaux_var canaux = fabCanImpl->_this();	// Creation de fabrique

	CosNaming::Name_var fabCanNom = new CosNaming::Name();
	fabCanNom->length(1);
	fabCanNom[0].kind = (const char*) "";
	char buffer[80];						// Le nom commence par 'C'

	int valide=0;
	unsigned int nbserveurs=0;
	while (!valide) {						// Cherche un nom non-utilise
		valide = 1;
		sprintf(buffer,"c%u",nbserveurs++);
		fabCanNom[0].id = (const char*) buffer;
		try {
			nsRef->bind(fabCanNom,canaux);
		} catch (CosNaming::NamingContext::AlreadyBound &) {	// Nom deja utilise
			valide = 0;
		} catch ( ... ) {	// Autres exceptions du serveur de noms
			cerr << "Erreur du service de nomage" << endl;
			return EXIT_FAILURE;
		}
	}

	// Publication de l'IOR de la fabrique de barres
	FabriqueDeBarres_impl * fabBarImpl = new FabriqueDeBarres_impl;
	FabriqueDeBarres_var barres = fabBarImpl->_this();		// Creation de fabrique

	CosNaming::Name_var fabBarNom = new CosNaming::Name();
	fabBarNom->length(1);
	fabBarNom[0].kind = (const char*) "";	// Le nom commence par 'B'

	valide=0;
	nbserveurs=0;
	while (!valide) {						// Cherche un nom non-utilise
		sprintf(buffer,"b%u",nbserveurs++);
		fabBarNom[0].id = (const char*) buffer;
		valide = 1;
		try {
			nsRef->bind(fabBarNom,barres);
		} catch (CosNaming::NamingContext::AlreadyBound &) {	// Nom deja utilise
			valide = 0;
		} catch ( ... ) {	// Autres exceptions du serveur de noms
			cerr << "Erreur du service de nomage" << endl;
			return EXIT_FAILURE;
		}
	}

	// Activation du serveur
	cout << "Fabrique de barres (" << fabBarNom[0].id << ") et fabrique de canaux ("
		 << fabCanNom[0].id << ") crees.\nAttente des requetes...\n";
	manager -> activate();
	orb -> run();

    // Supprimer les noms du naming service 
    nsRef->unbind(fabBarNom);
    nsRef->unbind(fabCanNom);

	return EXIT_SUCCESS;
}
